
-- Vendors

Vendor_Dialogue = {
	['ham_trainer'] = [[
		I can teach you some new [c yellow]skills[c white]... for a price of course!

		If you've mastered a skill, speak with the [c yellow]enchanter[c white] at the [c yellow]Femwood [c yellow]Library[c white] for more training.

		Also, my blind son can teach you how to wield more skills. You'll find him down in the [c yellow]cellar[c white].
	]],

	['ham_supplies'] = [[
		I have the finest supplies in all of... [c yellow]Hamlen[c white]!

		Of course, this map here ain't looking too hot. Say, why don't you visit my good friend [c yellow]Johm[c white] out by the [c yellow]Sandy [c yellow]Cave[c white]. He should be able to get it all fixed up for you.

		Oh, and don't forget to stock up on [c yellow]torches[c white]. You wouldn't want to get lost in a cave without one!
	]],

	['ham_weapons'] = [[
		Each one of my weapons has a unique [c yellow]power[c white].

		Make sure to match it up with the right [c yellow]armor [c yellow]set[c white] to fully unlock its abilities. Looks better, too!

		If you feel like it needs a good sharpening, head over to the town [c yellow]blacksmith[c white] to upgrade it.

		Don't forget to give the practice [c yellow]dummy[c white] a couple of whacks afterwards!
	]],

	['ham_armor'] = [[
		Buy a full [c yellow]set[c white] of armor today!

		Remember, your set's bonus is only as good as the [c yellow]lowest [c yellow]level[c white] piece.

		Head over to [c yellow]Sanna[c white] if you need a more experienced blacksmith.
	]],

	['ham_gambler'] = [[
		Trying to get into that [c yellow]graveyard[c white] by the swamp? My crazy brother [c yellow]Pete[c white] might still have the key. He likes to hang around that old withered [c yellow]tree[c white] behind the graveyard.

		Although, if you get lost looking for him, you could always try gambling for my spooky [c yellow]map[c white]! Heh heh!
	]],

	['cellar_vendor'] = [[
		Father has put me in the best [c yellow]room[c white] in the house!

		I can feel the warmth of the sun through the windows!

		I can hear the birds singing in the morning!

		What a wonderful place!

		[c unseen]...I can also hear the dead [c unseen]children clawing at the door at [c unseen]night... heh heh heh
	]],

	['dungeon_vendor'] = [[
		What year is it? How long have I been down here...

		Nevermind, let's see your coin.
	]],

	['fem_vendor'] = [[
		The [c yellow]Femwood [c yellow]Library[c white] contains a vast wealth of knowledge.

		Only the initiated may enter.

		Seek out [c yellow]Tas[c white] in the [c yellow]graveyard[c white]. You can usually find him reading alone in the closet. He enjoys visitors that bring him a good [c yellow]Mage [c yellow]Book[c white] and a [c yellow]Light[c white] spell.
	]],

	['library_gear'] = [[
		Why not replace those old tattered clothes with some new, enchanted ones?
	]],

	['san_supplies'] = [[
		Just some basic, [c yellow]dusty[c white] supplies. Nothing fancy here.

		Don't forget to tip your local [c yellow]beggar[c white]! Good things come to those who do.
	]],

	['san_trainer'] = [[
		Do you need to cut down your enemies swiftly and [c yellow]efficiently[c white]? Let me teach you some new powerful melee [c yellow]skills[c white].

		You should talk with [c yellow]Erb[c white] up in the old [c yellow]Ruined [c yellow]City[c white] if you really want to master your weapon arts.
	]],

	['san_weapons'] = [[
		These are the best weapons you'll find on this side of the river.

		Don't forget to pair it with a [c yellow]set[c white] of armor next door.
	]],

	['san_armor'] = [[
		Beware the old [c yellow]Sentinels[c white] in the [c yellow]Ruined [c yellow]City[c white]. They don't move fast but they sure pack a wallop!

		You're better off buying my armor to protect yourself against them. I promise to clean the dust off if you do!
	]],

	['graveyard_trainer'] = [[
		Have you seen ole' [c yellow]Rog[c white] wandering around here?

		He usually takes a break from the [c yellow]coffin[c white] life to scare unwary visitors and shoo away the occasional [c yellow]grave [c yellow]robber[c white].
	]],

	['hideout_vendor'] = [[
		What ya lookin' for? We have the best stolen- er, I mean found [c yellow]goods[c white] here!

		If you're really lookin' for the good stuff, we just got a brand new [c yellow]gambling[c white] machine!
	]],

	['lighthouse_vendor'] = [[
		Sure is nice up here, isn't it?

		You can even see a giant [c yellow]crow[c white] just south of here!
	]],

	['city_trainer'] = [[
		I was hoping I'd get out of here when the city collapsed. It's just my luck that I'm still trapped.
	]],

	['city_vendor'] = [[
		I still get a lot of business even though the city has been turned into rubble!
	]],

	['wym_vendor'] = [[
		Careful around the swamps here... that ain't your grandma's [c yellow]lava[c white] out there!

		[c yellow]Gurm[c white] is deep down into the swamp, and he's got the right [c yellow]tool[c white] for your survival. Bring him [c yellow]Steel [c yellow]Boots[c white] and plenty of [c yellow]Lava [c yellow]Sludge[c white] and he'll get you fixed right up.

		Buy my trusty [c yellow]map[c white] and you'll find him at the [c orange]orange[c white] mark.
	]],

	['wym_armor'] = [[
		The [c yellow]swamps[c white] here are like nothing you've ever seen. It'll slow you down while ghastly horrors come out at every corner!

		Are you sure you don't want to get some more [c yellow]experience[c white] first?
	]],

	['wym_weapons'] = [[
		If you think you're going into that swamp... well, at least equip some decent [c yellow]gear[c white] first!

		Get a [c yellow]map[c white] and proper supplies, then you might actually have a chance against that big ole' fat [c yellow]spider[c white]!
	]],

	['swamp_vendor'] = [[
		[c yellow]Gurm[c white] is pretty mad after I "borrowed" his legendary weapon. Eh, serves him right.

		Now if only I could get a hold of [c yellow]Jem's[c white] other ring...
	]],

	['chamber_vendor'] = [[
		The defeated, lost souls gather here in the [c yellow]Chamber[c white].

		You must defeat the mighty [c yellow]Guardians[c white] if you're ever to access the timeless [c yellow]Ancient[c white].
	]],

	['ancient_vendor'] = [[
		Time stands still in the [c yellow]Ancient[c white].

		You may have had faint [c yellow]memories[c white] of visiting here hundreds, thousands, or even millions of [c yellow]times[c white] before.
	]],

	['rebirth_vendor'] = [[
		Your [c yellow]coins[c white] don't last forever, so why not make your next [c yellow]journey[c white] easier?
	]],

	['evolve_vendor'] = [[
		To truly [c yellow]progress[c white], you must sometimes give up [c yellow]everything[c white] to acquire something barely out of reach.
	]],

	['transform_vendor'] = [[
		Is it coin that you're spending, or is it really just [c yellow]time[c white]?
	]],
}

-- Traders

Trader_Dialogue = {
	['soothing_salve'] = [[
		These [c yellow]slimy [c yellow]globs[c white] are quite effective for bandaging your bleeding wounds. I can never get enough of them since I'm always getting snagged on those damn [c yellow]thorns[c white] in the caves.

		If you're running low, I hear there's a big [c yellow]cave[c white] full of slimes somewhere.
	]],

	['firebomb'] = [[
		Are your weapons ineffective against the ghastly [c yellow]ghosts[c white] in the graveyard?

		Try lighting them up with these [c yellow]firebombs[c white].
	]],

	['cellar_key'] = [[
		Psst! Hey, over here!

		Have you seen that great big [c yellow]bat[c white] down in the caves?

		Well, sooner or later you'll need to get down into that dark [c yellow]cellar[c white], and I got your key right here!
	]],

	['throwing_knives'] = [[
		It's best to avoid the beaches around these parts... those [c yellow]crabs[c white] have mighty pincers!

		Of course, my [c yellow]throwing [c yellow]knives[c white] are of the finest quality and [c yellow]pierce[c white] right through their chitin.
	]],

	['poison'] = [[
		Tired of those pesky healers? Throw some of my [c yellow]poison[c white] at them!

		In fact, keep throwing cause the poison [c yellow]stacks[c white]! Ha ha!
	]],

	['nocturnal_map'] = [[
		Hey, help me out here!

		[c yellow]Seb[c white] says there's some sort of secret entrance to [c yellow]Raj's[c white] place but I can't seem to find it. I've been searching all over in the so-called [c yellow]Nocturnal [c yellow]Cave[c white].

		I'm running low on supplies, so maybe we can do a little trade for my [c yellow]map[c white], heh heh!
	]],

	['medium_torch'] = [[
		Put some good ole' [c yellow]slimy[c white] goop on a [c yellow]stick[c white] and you got yourself a pretty good [c yellow]torch[c white].

		Here, let me show you!
	]],

	['health_ring'] = [[
		Need some extra [c yellow]sustenance[c white] in your life? Put this ring on.

		Hell, put two of 'em on. You got two fingers, don't ya?
	]],

	['soothing_amulet'] = [[
		Wearing this [c yellow]amulet[c white] is like getting a neck massage all day long!
	]],

	['spider_sword'] = [[
		I've infused the power of [c yellow]spiders[c white] into this mighty [c yellow]sword[c white].

		I call it... the [c yellow]Spider [c yellow]Sword[c white]! Pretty good, right?

		Give those bastards a taste of their own medicine, ha ha!
	]],

	['better_map'] = [[
		Hey there, I'm [c yellow]Johm[c white]. Let me fix up that map for you. Careful handling that [c yellow]fire [c yellow]dust[c white] though... you don't want to get it in your eyes!

		I wish I hadn't lost my mountain climbing boots. Of course [c yellow]Sach[c white] has another pair if you're looking for some. Last I heard she's up north of [c yellow]Sanna[c white] by the [c yellow]Ruined [c yellow]City[c white].
	]],

	['flippers'] = [[
		Don't bother with that water by the [c yellow]Lighthouse[c white] without my stylish blue [c yellow]flippers[c white]!

		You'll never be [c yellow]slow[c white] in the water again!
	]],

	['sandy_map'] = [[
		Oh boy, mapping out that [c yellow]Sandy [c yellow]Cave[c white] was real fun, lemme tell ya! Hope you can read my [c red]scribbles[c white], ha ha!

		If you see [c yellow]Bursk[c white] in there, good luck trying to talk to him. He set up some sort of tomfoolery in that room.

		I fell through an [c yellow]illusory [c yellow]wall[c white] in another room. Couldn't remember where though. It lead to some [c yellow]slimy[c white] cavern from what I could tell.
	]],

	['slimy_map'] = [[
		You'll need [c yellow]Mountain [c yellow]Climbers[c white] or [c yellow]Flippers[c white] to get down to the [c yellow]Slimy [c yellow]Cave[c white]. Of course, I've heard of other sneaky ways in.

		The [c yellow]lava[c white] down there isn't so bad. Nothing like the [c yellow]lava[c white] over by [c yellow]Wymig[c white]!

		What you gotta watch out for are those damned evil [c yellow]venomous [c yellow]bats[c white], with their big gnarly [c yellow]fangs[c white]! They don't like hanging around water too much though.
	]],

	['poison_knives'] = [[
		What's better than the sound of flying death through the air? [c yellow]Poison-tipped[c white] flying death!
	]],

	['mountain_climbers'] = [[
		[c yellow]Johm[c white] sent ya, did he? Well, tell him he still owes me fifty game tokens!

		So you're looking to conquer some [c yellow]mountains[c white], eh? Or maybe just some small rocks...

		These will do the trick!
	]],

	['bone_wand'] = [[
		Command the dead and cripple your foes with deadly [c yellow]poison[c white]!
	]],

	['belligerent_potion'] = [[
		When you really want to piss off your enemies, a couple of drinks is all you need.

		Don't tell [c yellow]Biff[c white] I sometimes pour a little of this in his drink when he's not looking. Ha ha!
	]],

	['map_coin'] = [[
		Are you a [c yellow]collector[c white] like me?

		I love these little [c yellow]maps[c white]!
	]],

	['trophy_coin'] = [[
		Are you a [c yellow]collector[c white] like me?

		These little [c yellow]trophies[c white] are great!
	]],

	['key_coin'] = [[
		Are you a [c yellow]collector[c white] like me?

		These [c yellow]keys[c white] sure are something!
	]],

	['dark_potion'] = [[
		It's... much... too... bright... down... here...
	]],

	['tower_key'] = [[
		The [c yellow]Tower [c yellow]of [c yellow]Femwood[c white] is a very dangerous and haunted place.

		Try not to [c yellow]fall[c white] off the bridge at the top. It's not a very soft landing at the bottom!
	]],

	['invisibility_potion'] = [[
		Trying to sneak past a well-guarded area? Why not test one of my [c yellow]invisibility[c white] potions!

		Don't go running through [c yellow]water[c white] though. You'll splash it all over yourself and ruin the effect!
	]],

	['graveyard_key'] = [[
		Hey uh.. you wouldn't happen to have any [c yellow]swamp globs[c white], would ya? Heh heh! They help cool me down after a nice [c yellow]lava[c white] bath in the caves!

		I could trade ya this key for 'em, but I'd steer clear of the [c yellow]graveyard[c white] in any case. It's haunted I tell ya!

		If you are crazy enough to venture in, try visiting [c yellow]Rog's [c yellow]headstone[c white], wherever it was...
	]],

	['gash'] = [[
		Have you visited [c yellow]Raj's [c yellow]Hideout[c white] yet?

		We have the best gambling around. You can usually pick off a few tokens from all those damn [c yellow]goblins[c white] running around down there.
	]],

	['dark_ring'] = [[
		The [c yellow]graveyard[c white] sure is a good place to find corpses, isn't it?

		Make sure to pay my [c yellow]headstone[c white] a visit. Heh heh!
	]],

	['library_key'] = [[
		What? Who goes there? Oh...

		Have you got another [c yellow]book[c white] for me? I'm almost done reading this one.

		It gets mighty chilly in here. If only I could get another warming torch.
	]],

	['ankh'] = [[
		Give new [c yellow]life[c white] from the remains of another!
	]],

	['bleed_ring'] = [[
		Be careful down here... you've got to deal with [c yellow]thorns[c white], bats, and spiders for starters. Not to mention it's pitch dark!

		Even worse, there's a big red [c yellow]bat[c white] flying around somewhere in this cave!

		Put on a couple of these [c yellow]rings[c white] and you might have a chance, heh heh.
	]],

	['potion_of_reach'] = [[
		[c yellow]Rog's[c white] headstone keeps falling down from the [c yellow]graveyard[c white], and the townsfolk keep putting up new ones!

		Maybe they should put it on the [c yellow]left[c white] side of the graveyard, where there isn't such a massive [c yellow]hole[c white] in the ground.
	]],

	['dimensionality'] = [[
		Ever wanted to slip through the cracks, [c yellow]literally[c white]?

		With this [c yellow]relic[c white] you can get out of, or into, any tight situation!
	]],

	['rajs_hiding_place'] = [[
		Ready to meet the big boss, [c yellow]Raj[c white]?

		Well, too bad, he doesn't like to be found!

		I will say though, there's a secret entrance that doesn't require any fancy [c yellow]slippers[c white] or [c yellow]relics[c white].

		In fact, it's somewhere in this [c yellow]cave[c white]!
	]],

	['shrapnel_bomb'] = [[
		Want to test out my new [c yellow]invention[c white]?

		I've been toying around with these [c yellow]Firebombs[c white], trying to come up with new ways to slice through my enemies' [c yellow]defenses[c white]!

		Just make sure to throw it away from [c yellow]yourself[c white], heh heh!
	]],

	['dungeon_key'] = [[
		Have you visited the dungeon? Let's just say it's pretty... [c yellow]dank[c white].

		If you're looking for some shortcuts, it's got easy access to all sorts of [c yellow]caves[c white]. You can even get to [c yellow]Femwood[c white] down there!

		Shh! Don't tell anyone about that one though, heh heh.
	]],

	['blade_dance'] = [[
		Want to learn the ways of master [c yellow]Raj[c white] from me?

		No? Well, you could always ask him nicely! Ha ha!
	]],

	['bloodletter'] = [[
		There's nothing like bathing in the [c yellow]blood[c white] of your fallen enemies, heh heh!
	]],

	['evasion'] = [[
		I can teach you the secrets of [c yellow]dodging[c white] any attack... or other people for that matter.
	]],

	['battle_potion'] = [[
		Ever wanted to just get into a [c yellow]fight[c white] without having to shuffle around to agitate all those monsters?

		Try my special brew. You can [c yellow]stand [c yellow]still[c white] and let the monsters come right to you!

		Use it in [c yellow]lava[c white] and you'll see what I mean!
	]],

	['lightning_ring'] = [[
		If you don't end up getting lost in here forever, these rings might give you a chance against the [c yellow]King's[c white] powerful [c yellow]pincer [c yellow]attack[c white].
	]],

	['chain_lightning'] = [[
		Harness the energy of the one true [c yellow]King[c white] and unlock ultimate power!
	]],

	['toughness'] = [[
		Nothing beats good ole' fashioned [c yellow]thick [c yellow]skin[c white], and you're going to need lots in this old [c yellow]Sandy [c yellow]Cave[c white]!

		Beware, the mighty [c yellow]King[c white] lives somewhere in this maze of a cave. Make no mistake, he'll pick you up and [c yellow]pincer[c white] you in half like you were nothing!
	]],

	['trolls_ring'] = [[
		Alright, you got me!

		I'm sure [c yellow]Biff[c white] over in [c yellow]Sanna[c white] would love this [c yellow]ring[c white]. Make sure to dangle it in front of him for a while before trading it. I love seeing him get worked up!
	]],

	['beggar'] = [[
		Boy, sure is hot out here. I could really go for a drink right about now...

		Spare a [c yellow]token[c white] and help an old fella out?
	]],

	['lucky_amulet'] = [[
		Are you ready to take your [c yellow]gambling[c white] addiction to the next level?

		Nah, I'm just jokin' around. But seriously, you should try out my [c yellow]Lucky [c yellow]Amulet[c white].

		I never leave home without it!
	]],

	['witch_key'] = [[
		Hey I got a key for ya! Well, it's not the [c yellow]City [c yellow]key[c white], but it's really good, I promise!

		Talk to old [c yellow]Sae[c white] in the [c yellow]Lighthouse[c white] if you're thinking about exploring these ruins.
	]],

	['potion_of_haste'] = [[
		Ever felt the need to [c yellow]jolt[c white] your party into action? Slip this one into their tea, heh heh!

		Nevermind the [c yellow]color[c white]... it's real good, I swear!
	]],

	['fracture'] = [[
		I got myself into a bind here when this building collapsed, and now I don't have any way to break free!
	]],

	['unstable_potion'] = [[
		I was banished from the [c yellow]library[c white] because of my experimental magic.

		When I get out of this cage, I'll show them!
	]],

	['venom_blade'] = [[
		If you're heading up to the [c yellow]Ruined [c yellow]City[c white], you best prepare yourself. Those [c yellow]Ants[c white] are mighty smart and only go for the weaklings in your group.

		If you want to tilt the odds, use my [c yellow]Venom [c yellow]Blade[c white] against those damned [c yellow]Skeleton [c yellow]Priests[c white]!
	]],

	['taunt'] = [[
		There's nothing I hate more in this world than a dumb [c yellow]mage[c white], with their dumb hat and their dumb wand!

		That's why I love [c yellow]aggravating[c white] 'em and making em' forgot all about their dumb spells! Ha ha!
	]],

	['arcane_cider'] = [[
		The mages around here sure are thirsty for their [c yellow]mana[c white].
	]],

	['brilliant_amulet'] = [[
		The [c yellow]Queen[c white] used to live high up in this dark [c yellow]tower[c white].

		Not many have made it past the bridge to see if she's still there.
	]],

	['magic_ring'] = [[
		The first step to becoming a [c yellow]mage[c white] is to, well, [c yellow]dress[c white] like one.
	]],

	['cold_ring'] = [[
		Welcome to the [c yellow]Femwood [c yellow]Library[c white].

		I assumed you showed your library card at the door, yes? Ha ha!

		We have lots of books here if [c yellow]arcane[c white] knowledge is what you seek... and of course the occasional [c yellow]bat[c white] swooping around.
	]],

	['fireball'] = [[
		Forget about [c yellow]cold[c white] or [c yellow]lightning[c white] spells... sometimes you have to [c yellow]incinerate[c white] the whole room!
	]],

	['ghostly_potion'] = [[
		If you really want to sneak around up here, you'll want the best potions... something that makes you [c yellow]ghostly[c white]!

		Of course if you can't handle the [c yellow]heat[c white], should you really be mucking around in the kitchen?
	]],

	['ice_nova'] = [[
		Harness the energy of the one true [c yellow]Queen[c white] and unlock ultimate power!
	]],

	['glowing_ring'] = [[
		Ah, another visitor to this [c yellow]desolate[c white] place.

		Have you come to pay tribute to the [c yellow]Dead [c yellow]Queen[c white]?
	]],

	['blue_runestone'] = [[
		...
	]],

	['fire_blast'] = [[
		The [c yellow]Queen[c white] taught me this magic long ago.

		Go on... you'll find her at her throne.
	]],

	['elusive_potion'] = [[
		I've seen hundreds of adventurers fall off this old rickety [c yellow]bridge[c white] in my time. Never gets old, heh heh!

		Although, sometimes they walk out onto that [c yellow]spot[c white] over there and just vanish into thin air! No screaming or nothing!

		Never really wanted to try it myself, though.
	]],

	['hand_of_zog'] = [[
		Few have made it here. Fewer have faced [c yellow]Zog[c white] and lived to tell the tale.

		He's commanded armies and decimated worlds in other planes.

		Now's your chance for a taste of true [c yellow]power[c white].
	]],

	['hungering_ring'] = [[
		Ah... you must have found my note.

		Are you ready to give up your life for ultimate [c yellow]power[c white]?
	]],

	['passage_key'] = [[
		Aren't you just [c blood]dying[c white] to see what's behind this mysterious door? Heh heh...
	]],

	['black_runestone'] = [[
		...
	]],

	['cunning_amulet'] = [[
		Thieves can be quite devious when it comes to their [c yellow]equipment[c white].
	]],

	['magic_barrier'] = [[
		What's a good [c yellow]offense[c white] without a little bit of [c yellow]defense[c white]?
	]],

	['fire_ring'] = [[
		Sooner or later, you'll need to splash around in hot [c yellow]lava[c white], gathering up all that [c yellow]sludge[c white] and what not.

		Trust me, you're going to want to have these wonderful [c yellow]rings[c white].
	]],

	['poison_ring'] = [[
		Poison can be quite deadly in these caves, especially if you don't [c yellow]treat[c white] it right away with a bat [c yellow]fang[c white].

		I'd stick to the [c yellow]water[c white] if you know what's good for ya!
	]],

	['warm_ring'] = [[
		Mmm... nice and [c yellow]warm[c white] over here!

		Put these on and you'll be even [c yellow]warmer[c white].
	]],

	['warding_amulet'] = [[
		Protection from the [c yellow]elements[c white] is crucial.

		Without it, how will you ever defeat [c yellow]Zog[c white]?
	]],

	['fishermans_necklace'] = [[
		I don't know what it is, but whenever I wear this [c yellow]necklace[c white], more things just turn up out of nowhere.
	]],

	['potion_of_unlearning'] = [[
		Sometimes you just need to drink yourself into a stupor to forget about all your past [c yellow]mishaps[c white].
	]],

	['city_key'] = [[
		Defeat the [c yellow]Lesser [c yellow]Guardians[c white] and you may enter the old [c yellow]City[c white].

		Beware the [c yellow]Four [c yellow]Sentinels[c white]. They guard the powers of the ancient [c yellow]Runestones[c white].
	]],

	['cloudy_potion'] = [[
		Thinking about taking the fast way down?

		Drink my [c yellow]potion[c white] beforehand... or don't. I don't care.

		Either way, you won't feel a thing!
	]],

	['bright_potion'] = [[
		It's... much... too... dark... up... here...
	]],

	['large_torch'] = [[
		I got the best [c yellow]torches[c white] you can find... well, for lighting up the place at least.
	]],

	['rupture'] = [[
		Why bury the dead when you can watch their corpse [c yellow]explode[c white]?
	]],

	['ring_of_pain'] = [[
		Is your life getting to be too [c yellow]easy[c white]?

		Why not kick it up a notch with these lovely [c yellow]rings[c white]?
	]],

	['building_key'] = [[
		Let me know if you find any more of these shiny little fragments.

		Maybe someday I can piece them together and make a truly valuable [c yellow]trinket[c white]!
	]],

	['flay'] = [[
		Want to really do some damage with the elements? Strip off all that tough [c yellow]hide[c white] and get down to the raw flesh!
	]],

	['red_runestone'] = [[
		...
	]],

	['flamuss'] = [[
		Well done on passing the trial.

		You have been granted audience to the legendary [c yellow]Flamuss[c white]!
	]],

	['mana_shield'] = [[
		After teaching this ancient magic to [c yellow]Zog[c white], I was banished and imprisoned here forever!
	]],

	['pickpocket'] = [[
		Who doesn't like an extra bit of [c yellow]coin[c white]?

		The real deal though is [c yellow]hunting[c white] other adventurers, heh heh.

		Talk to [c yellow]Lan[c white] if you're up for it. She's usually hiding in the [c yellow]Chamber[c white], waiting for unsuspecting victims.
	]],

	['lava_potion'] = [[
		[c yellow]Gurm[c white] threw me in here because he didn't want me undercutting his business!

		Once I pick this lock I'll send a [c yellow]Moon [c yellow]Blade[c white] right through him!
	]],

	['green_runestone'] = [[
		...
	]],

	['winged_shoes'] = [[
		Want to learn how to descend like a [c yellow]spider[c white]?

		Well, I can't teach you that, but you can try on these magical [c yellow]shoes[c white] instead!

		Just, uh, make sure to lace them up real tight beforehand. That certainly would've saved [c yellow]Rog[c white] the first time, ha ha!
	]],

	['mythical_amulet'] = [[
		Embrace the power created by [c yellow]Zog[c white]!
	]],

	['fire_walkers'] = [[
		What do you want? Should I throw you in a [c yellow]cage[c white] like the rest of 'em?

		Well then hand over the [c yellow]goods[c white] or scram!
	]],

	['constricting_amulet'] = [[
		Good things come to those who endure the [c yellow]pain[c white]!
	]],

	['silver_amulet'] = [[
		[c yellow]Time[c white] is precious and must not be wasted.
	]],

	['greater_battle_potion'] = [[
		I've been mixing all these different potions together and I think I've got another winner.

		Why not have a little drink before your meeting with [c yellow]Jem[c white]?
	]],

	['lava_boots'] = [[
		Some say if [c orange]Sol[c white] gets banished, the lava will go away.

		Of course, that wouldn't be good for [c yellow]business[c white]!
	]],

	['diffusion_amulet'] = [[
		I wonder where [c orange]Sol[c white] is now?

		That bastard really shook things up. It really hasn't been the same with all this [c yellow]lava[c white] everywhere.
	]],

	['warming_torch'] = [[
		Whoa now, hold it there buddy!

		That over there ain't your grandpa's [c yellow]ice[c white]!

		You sure you don't want my trusty [c yellow]torch[c white] before you become another [c yellow]icy[c white] [c yellow]statue[c white]?
	]],

	['swamp_key'] = [[
		Hmm, access to the best [c yellow]shortcut[c white] around or the mighty ring of [c yellow]Jem[c white]?

		Decisions, decisions...
	]],

	['hunt'] = [[
		Shh... don't let anyone know I'm here, or else you might find a nice [c yellow]blade[c white] in your back!
	]],

	['scorching_amulet'] = [[
		The townsfolk say [c orange]Sol[c white] spewed out all this lava you see here today.

		At least I can make some [c yellow]coin[c white] off it all, ha ha!
	]],

	['chilly_amulet'] = [[
		Legend has it that [c orange]Sol[c white] absorbed all the energy down here, leaving it frozen.

		Must be why you see such a strange dichotomy in close proximity.
	]],

	['icebrand'] = [[
		This [c light_blue]sword[c white] must have been forged when [c orange]Sol[c white] came and laid waste to the mountain side.

		Who knows how long it's been around.
	]],

	['ultimate_battle_potion'] = [[
		I've managed to capture the essence of a [c yellow]Guardian's[c white] [c yellow]soul[c white]!

		Don't wait too long to open it though. [c yellow]Souls[c white] are better fresh!
	]],

	['mystic_staff'] = [[
		Will I ever meet the [c yellow]Elusive [c yellow]One[c white]?

		I've dedicated my life to his teachings...
	]],

	['icy_key'] = [[
		Hopefully you brought your [c yellow]Winged [c yellow]Shoes[c white], because the [c yellow]chasms[c white] around here are much deeper than they look!

		There's a [c yellow]mage[c white] down below somewhere... must have been banished, or maybe just likes the solitude.
	]],

	['empower'] = [[
		Have you visited the [c yellow]Ancient[c white]? It's a place where time seemingly does not exist...

		[c yellow]Zog[c white] lives in a dimension that can only be accessed from there.

		Can he ever be defeated, though? Is he banished, only to return later?
	]],

	['ice_walkers'] = [[
		These can't protect you against those [c yellow]Icy[c white] [c yellow]Statues[c white], but at least you won't turn into one!
	]],

	['hasten'] = [[
		It's [c light_blue]cold[c white] and desolate down here.

		Just the way I like it.
	]],

	['voidstone'] = [[
		Step into the [c unseen]void[c white] and be released.
	]],

	['greater_ring_of_pain'] = [[
		Ah [c yellow]suffering[c white], my old friend.

		Are you ready to see how deep the [c yellow]pain[c white] goes?
	]],

	['hangmans_noose'] = [[
		There's nothing left
		[c unseen]but [c unseen]suffering.[c white]

		Wrap it around...
		[c unseen]and [c unseen]tighten.[c white]

		Just like that...
		[c unseen]keep [c unseen]tightening![c white]

		Almost there...
		[c unseen]keep... [c unseen]tighten... [c unseen]ing...[c white]
	]],

	['bottle_of_tears'] = [[
		Miss the [c blood]Blood[c white] [c blood]Moon[c white] already?

		Me too...

		I wish every night was [c blood]bloody[c white]!
	]],

	['potion_of_regret'] = [[
		Have you made some bad [c yellow]decisions[c white] lately?

		No worries.

		Drink up!
	]],

	['rite_of_soul'] = [[
		Do you really think you've come to the end of your [c yellow]journey[c white]?

		I think it's just begun!
	]],

	['final_key'] = [[
		When the moon turns full [c blood]blood[c white], you'll know it's time.
	]],

	['rite_of_evolution'] = [[
		[c yellow]Progressing[c white], are we?

		I'm sure you'll be back again, and again, and again, and again...
	]],

	['ultimate_key'] = [[
		Are you ready for the [c yellow]Ultimate[c white] [c yellow]Chores[c white]?
	]],
}
